////////////////////////////////////////////////////////////////////////////
//// EA-DOGM kijelz driver												////
//// The following have special meaning: 								////
//// \f Clear display 													////
//// \n Go to start of second line 										////
//// \b Move back one position 											////
//// \c Cursor on 														////
//// \d Blink character 												////
//// 																	////
//// lcd_gotoxy(x,y) Set write position on LCD (upper left is 1,1)      ////
//// Topor Zoltn - 2007 - http://www.hobbielektronika.hu/			    ////
////////////////////////////////////////////////////////////////////////////

//--- ORIGINAL LCD
//     D0  enable
//     D1  rs
//     D2  rw

//     D4  D4
//     D5  D5
//     D6  D6
//     D7  D7

#ifndef LCD_E
#define LCD_E PIN_D0
#define LCD_RS PIN_D1
#define LCD_RW PIN_D2
#define LCD_DB4 PIN_D4
#define LCD_DB5 PIN_D5
#define LCD_DB6 PIN_D6
#define LCD_DB7 PIN_D7
#endif

// Clear and home
#define LCD_CLR_DISP 0x01 // Clear screen, home cursor, unshift display
#define LCD_RETURN_HOME 0x02 // Home cursor, unshift display

#define LCD_DISP_OFF 0x08 // Display off
#define LCD_DISP_ON  0x0C // Display on, cursor off
#define LCD_DISP_ON_BLINK 0x0D // Display on, cursor off, blink char
#define LCD_DISP_ON_CURSOR 0x0E // Display on, cursor on
#define LCD_DISP_ON_CURSOR_BLINK 0x0F // Display on, cursor on, blink char

// Function set: set interface data length and number of display lines
#define LCD_FUNCTION_4BIT_1LINE  0b00100001 // 4-bit interface, single line, 5x7 dots
#define LCD_FUNCTION_4BIT_2LINES 0b00101001 // 4-bit interface, dual line, 5x7 dots
#define LCD_FUNCTION_8BIT_1LINE  0b00110001 // 8-bit interface, single line, 5x7 dots
#define LCD_FUNCTION_8BIT_2LINES 0b00111001 // 8-bit interface, dual line, 5x7 dots

#define LCD_CGRAM_BASE_ADDR 0x40 // Set the CGRAM address
#define LCD_DDRAM_BASE_ADDR 0x80 // Set the DDRAM address

// Address positions
#define LCD_LINE_1 0x80 // Position of start of line 1
#define LCD_LINE_2 0xC0 // Position of start of line 2
#define LCD_LINE_3 0x94 // Position of start of line 3
#define LCD_LINE_4 0xD4 // Position of start of line 4

BYTE const LCD_INIT_STRING[8] = {
LCD_FUNCTION_4BIT_2LINES, // Set mode: 4-bit md 2 sor normal font 1-es INSTL
0b00011100,	//Bias set
0b01010010,	//Power control	5V
0b01101001, //Follower control
0b01110100,	//Contrast set
LCD_DISP_ON,	//Display ON
LCD_CLR_DISP,	//Clear Display
0b00000110	//Entry mode cursor auto increment
};

BYTE const LCD_CUSTOM_CHARS[] = {
//0x04,0x1F,0x11,0x11,0x11,0x11,0x11,0x1F, 
//0x04,0x1F,0x11,0x11,0x11,0x11,0x1F,0x1F, 
//0x04,0x1F,0x11,0x11,0x11,0x1F,0x1F,0x1F, 
//0x04,0x1F,0x11,0x11,0x1F,0x1F,0x1F,0x1F, 
//0x04,0x1F,0x11,0x1F,0x1F,0x1F,0x1F,0x1F, 
//0x04,0x1F,0x1F,0x1F,0x1F,0x1F,0x1F,0x1F, 

//0x1F,0x1F,0x1F,0x1F,0x1F,0x1F,0x1F,0x1F, 
//0x1F,0x1F,0x1F,0x1F,0x1F,0x1F,0x1F,0x1F
}; 

BYTE lcd_line;

////////////////////////////////////////////////////////////////////////////
BYTE lcd_read_nibble() {
////////////////////////////////////////////////////////////////////////////
BYTE retval;

#bit retval_0 = retval.0
#bit retval_1 = retval.1
#bit retval_2 = retval.2
#bit retval_3 = retval.3
retval = 0;
output_high(LCD_E);
delay_cycles(1);
retval_0 = input(LCD_DB4);
retval_1 = input(LCD_DB5);
retval_2 = input(LCD_DB6);
retval_3 = input(LCD_DB7);
output_low(LCD_E);
return(retval);
}

////////////////////////////////////////////////////////////////////////////
BYTE lcd_read_byte() {
////////////////////////////////////////////////////////////////////////////
BYTE low,high;

output_high(LCD_RW);
delay_cycles(1);
high = lcd_read_nibble();
low = lcd_read_nibble();
return((high << 4) | low);
}

////////////////////////////////////////////////////////////////////////////
void lcd_send_nibble( BYTE n ) {
////////////////////////////////////////////////////////////////////////////
output_bit(LCD_DB4, !!(n & 1));
output_bit(LCD_DB5, !!(n & 2));
output_bit(LCD_DB6, !!(n & 4));
output_bit(LCD_DB7, !!(n & 8));
delay_cycles(1);
output_high(LCD_E);
delay_us(2);
output_low(LCD_E);
}

////////////////////////////////////////////////////////////////////////////
void lcd_send_byte( BYTE address, BYTE n ) {
////////////////////////////////////////////////////////////////////////////
output_low(LCD_RS);
while (bit_test(lcd_read_byte(),7)); // wait until busy flag is low
if (address)
output_high(LCD_RS);
else
output_low(LCD_RS);

delay_cycles(1);
output_low(LCD_RW);
delay_cycles(1);
output_low(LCD_E);
lcd_send_nibble(n >> 4);
lcd_send_nibble(n & 0xf);
}

////////////////////////////////////////////////////////////////////////////
void lcd_init() {
////////////////////////////////////////////////////////////////////////////
BYTE i;
lcd_line = 1;

output_low(LCD_RS);
output_low(LCD_RW);
output_low(LCD_E);
delay_ms(15);
for (i=1;i<=3;++i) {
lcd_send_nibble(3);
delay_ms(5);
}
lcd_send_nibble(2);

for (i=0;i<sizeof(LCD_INIT_STRING);i++) {
	lcd_send_byte(0, LCD_INIT_STRING[i]);
}
}

////////////////////////////////////////////////////////////////////////////
void lcd_init_custom_chars() {
////////////////////////////////////////////////////////////////////////////
BYTE i;

lcd_send_byte(0,LCD_CGRAM_BASE_ADDR);
for (i=0;i<64;i++) {
lcd_send_byte(1,LCD_CUSTOM_CHARS[i]);
delay_ms(2);
}
}

////////////////////////////////////////////////////////////////////////////
void lcd_gotoxy( BYTE x, BYTE y ) {
////////////////////////////////////////////////////////////////////////////
BYTE address;

switch(y) {
case 1 : address=LCD_LINE_1; break;
case 2 : address=LCD_LINE_2; break;
case 3 : address=LCD_LINE_3; break;
case 4 : address=LCD_LINE_4; break;
}
address+=x-1;
lcd_send_byte(0,0x80 | address);
}

////////////////////////////////////////////////////////////////////////////
void lcd_putc( char c ) {
////////////////////////////////////////////////////////////////////////////
switch(c) {
case '\f' : lcd_send_byte(0,LCD_CLR_DISP);
lcd_line=1;
delay_ms(2); break;
case '\n' : lcd_gotoxy(1,++lcd_line); break;
//case '\c' : lcd_send_byte(0,LCD_DISP_ON_CURSOR); break;
//case '\d' : lcd_send_byte(0,LCD_DISP_ON_BLINK); break;
default : lcd_send_byte(1,c); break;
}
}

////////////////////////////////////////////////////////////////////////////
char lcd_getc( BYTE x, BYTE y ) {
////////////////////////////////////////////////////////////////////////////
char value;

lcd_gotoxy(x,y);
while (bit_test(lcd_read_byte(),7)); // wait until busy flag is low
output_high(LCD_RS);
value = lcd_read_byte();
output_low(LCD_RS);
return(value);
}
